@@ -6,24 +6,27 @@
  *                                                                          *
  *                          C Implementation File                           *
  *                                                                          *
- *          Copyright (C) 1992-2003 Free Software Foundation, Inc.          *
+ *          Copyright (C) 1992-2005, 2007 Free Software Foundation, Inc.    *
  *                                                                          *
  * GNAT is free software;  you can  redistribute it  and/or modify it under *
  * terms of the  GNU General Public License as published  by the Free Soft- *
- * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * ware  Foundation;  either version 3,  or (at your option) any later ver- *
  * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
  * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
- * for  more details.  You should have  received  a copy of the GNU General *
- * Public License  distributed with GNAT;  see file COPYING.  If not, write *
- * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
- * MA 02111-1307, USA.                                                      *
+ * for  more details.  You should have received a copy of the GNU General   *
+ * Public License along with GCC; see the file COPYING3.  If not see        *
+ * <http://www.gnu.org/licenses/>.                                          *
  *                                                                          *
  * GNAT was originally developed  by the GNAT team at  New York University. *
  * Extensive contributions were provided by Ada Core Technologies Inc.      *
  *                                                                          *
  ****************************************************************************/
 
+#ifdef VMS
+#define _POSIX_EXIT 1
+#endif
+
 #include "config.h"
 #include "system.h"
 
@@ -32,13 +35,6 @@
 #endif
 #include "adaint.h"
 
-#ifdef VMS
-#ifdef exit
-#undef exit
-#endif
-#define exit __posix_exit
-#endif
-
 /* These can be set by command line arguments */
 char *binder_path = 0;
 char *linker_path = 0;
@@ -138,7 +134,7 @@ process_args (int *p_argc, char *argv[])
 
       if (! strcmp (argv[i], "-gnatbind"))
 	{
-	  /* Explicit naming of binder.  Grab the value then remove the
+	  /* Explicit naming of binder. Grab the value then remove the
 	     two arguments from the argument list. */
 	  if ( i + 1 >= *p_argc )
 	    {
