@@ -1,6 +1,7 @@
 /* Calculate branch probabilities, and basic block execution counts.
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
+   2000, 2001, 2002, 2003, 2004, 2005, 2007
+   Free Software Foundation, Inc.
    Contributed by James E. Wilson, UC Berkeley/Cygnus Support;
    based on some ideas from Dain Samples of UC Berkeley.
    Further mangling by Bob Manson, Cygnus Support.
@@ -9,7 +10,7 @@ This file is part of GCC.
 
 GCC is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2, or (at your option) any later
+Software Foundation; either version 3, or (at your option) any later
 version.
 
 GCC is distributed in the hope that it will be useful, but WITHOUT ANY
@@ -18,9 +19,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with GCC; see the file COPYING.  If not, write to the Free
-Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.  */
+along with GCC; see the file COPYING3.  If not see
+<http://www.gnu.org/licenses/>.  */
 
 /* Generate basic block profile instrumentation and auxiliary files.
    Profile generation is optimized, so that not all arcs in the basic
@@ -72,12 +72,6 @@ Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 /* Hooks for profiling.  */
 static struct profile_hooks* profile_hooks;
 
-/* File for profiling debug output.  */
-static inline FILE*
-profile_dump_file (void) {
-  return profile_hooks->profile_dump_file ();
-}
-
 /* Additional information about the edges we need.  */
 struct edge_info {
   unsigned int count_valid : 1;
@@ -226,7 +220,6 @@ instrument_values (histogram_values values)
 	  gcc_unreachable ();
 	}
     }
-  VEC_free (histogram_value, heap, values);
 }
 
 
@@ -471,7 +464,7 @@ compute_branch_probabilities (void)
 	}
     }
   if (dump_file)
-    dump_flow_info (dump_file);
+    dump_flow_info (dump_file, dump_flags);
 
   total_num_passes += passes;
   if (dump_file)
@@ -496,7 +489,6 @@ compute_branch_probabilities (void)
     {
       edge e;
       edge_iterator ei;
-      rtx note;
 
       if (bb->count < 0)
 	{
@@ -531,7 +523,7 @@ compute_branch_probabilities (void)
 	{
 	  FOR_EACH_EDGE (e, ei, bb->succs)
 	    e->probability = (e->count * REG_BR_PROB_BASE + bb->count / 2) / bb->count;
-	  if (bb->index >= 0
+	  if (bb->index >= NUM_FIXED_BLOCKS
 	      && block_ends_with_condjump_p (bb)
 	      && EDGE_COUNT (bb->succs) >= 2)
 	    {
@@ -552,37 +544,9 @@ compute_branch_probabilities (void)
 		index = 19;
 	      hist_br_prob[index]++;
 
-	      /* Do this for RTL only.  */
-	      if (!ir_type ())
-		{
-		  note = find_reg_note (BB_END (bb), REG_BR_PROB, 0);
-		  /* There may be already note put by some other pass, such
-		     as builtin_expect expander.  */
-		  if (note)
-		    XEXP (note, 0) = GEN_INT (prob);
-		  else
-		    REG_NOTES (BB_END (bb))
-		      = gen_rtx_EXPR_LIST (REG_BR_PROB, GEN_INT (prob),
-					   REG_NOTES (BB_END (bb)));
-		}
 	      num_branches++;
 	    }
 	}
-      /* Otherwise try to preserve the existing REG_BR_PROB probabilities
-         tree based profile guessing put into code.  BB can be the
-	 ENTRY_BLOCK, and it can have multiple (fake) successors in
-	 EH cases, but it still has no code; don't crash in this case.  */
-      else if (profile_status == PROFILE_ABSENT
-	       && !ir_type ()
-	       && EDGE_COUNT (bb->succs) > 1
-	       && BB_END (bb)
-	       && (note = find_reg_note (BB_END (bb), REG_BR_PROB, 0)))
-	{
-	  int prob = INTVAL (XEXP (note, 0));
-
-	  BRANCH_EDGE (bb)->probability = prob;
-	  FALLTHRU_EDGE (bb)->probability = REG_BR_PROB_BASE - prob;
-	}
       /* As a last resort, distribute the probabilities evenly.
 	 Use simple heuristics that if there are normal edges,
 	 give all abnormals frequency of 0, otherwise distribute the
@@ -609,7 +573,7 @@ compute_branch_probabilities (void)
 	      FOR_EACH_EDGE (e, ei, bb->succs)
 		e->probability = REG_BR_PROB_BASE / total;
 	    }
-	  if (bb->index >= 0
+	  if (bb->index >= NUM_FIXED_BLOCKS
 	      && block_ends_with_condjump_p (bb)
 	      && EDGE_COUNT (bb->succs) >= 2)
 	    num_branches++, num_never_executed;
@@ -693,8 +657,7 @@ compute_value_histograms (histogram_values values)
 
       hist->hvalue.next = ann->histograms;
       ann->histograms = hist;
-      hist->hvalue.counters = 
-	    xmalloc (sizeof (gcov_type) * hist->n_counters);
+      hist->hvalue.counters =  XNEWVEC (gcov_type, hist->n_counters);
       for (j = 0; j < hist->n_counters; j++)
 	hist->hvalue.counters[j] = aact_count[j];
     }
@@ -704,7 +667,9 @@ compute_value_histograms (histogram_values values)
       free (histogram_counts[t]);
 }
 
-#define BB_TO_GCOV_INDEX(bb)  ((bb)->index + 1)
+/* The entry basic block will be moved around so that it has index=1,
+   there is nothing at index 0 and the exit is at n_basic_block.  */
+#define BB_TO_GCOV_INDEX(bb)  ((bb)->index - 1)
 /* When passed NULL as file_name, initialize.
    When passed something else, output the necessary commands to change
    line to LINE and offset to FILE_NAME.  */
@@ -917,7 +882,7 @@ branch_prob (void)
 	num_instrumented++;
     }
 
-  total_num_blocks += n_basic_blocks + 2;
+  total_num_blocks += n_basic_blocks;
   if (dump_file)
     fprintf (dump_file, "%d basic blocks\n", n_basic_blocks);
 
@@ -938,7 +903,7 @@ branch_prob (void)
       gcov_position_t offset;
 
       offset = gcov_write_tag (GCOV_TAG_BLOCKS);
-      for (i = 0; i != (unsigned) (n_basic_blocks + 2); i++)
+      for (i = 0; i != (unsigned) (n_basic_blocks); i++)
 	gcov_write_unsigned (0);
       gcov_write_length (offset);
     }
@@ -946,7 +911,7 @@ branch_prob (void)
    /* Keep all basic block indexes nonnegative in the gcov output.
       Index 0 is used for entry block, last index is for exit block.
       */
-  ENTRY_BLOCK_PTR->index = -1;
+  ENTRY_BLOCK_PTR->index = 1;
   EXIT_BLOCK_PTR->index = last_basic_block;
 
   /* Arcs */
@@ -977,8 +942,7 @@ branch_prob (void)
 		    flag_bits |= GCOV_ARC_FALLTHROUGH;
 		  /* On trees we don't have fallthru flags, but we can
 		     recompute them from CFG shape.  */
-		  if (ir_type ()
-		      && e->flags & (EDGE_TRUE_VALUE | EDGE_FALSE_VALUE)
+		  if (e->flags & (EDGE_TRUE_VALUE | EDGE_FALSE_VALUE)
 		      && e->src->next_bb == e->dest)
 		    flag_bits |= GCOV_ARC_FALLTHROUGH;
 
@@ -994,116 +958,57 @@ branch_prob (void)
   /* Line numbers.  */
   if (coverage_begin_output ())
     {
+      gcov_position_t offset;
+
       /* Initialize the output.  */
       output_location (NULL, 0, NULL, NULL);
 
-      if (!ir_type ())
+      FOR_EACH_BB (bb)
 	{
-	  gcov_position_t offset;
+	  block_stmt_iterator bsi;
 
-	  FOR_EACH_BB (bb)
+	  offset = 0;
+
+	  if (bb == ENTRY_BLOCK_PTR->next_bb)
 	    {
-	      rtx insn = BB_HEAD (bb);
-	      int ignore_next_note = 0;
+	      expanded_location curr_location = 
+		expand_location (DECL_SOURCE_LOCATION (current_function_decl));
+	      output_location (curr_location.file, curr_location.line,
+			       &offset, bb);
+	    }
 
-	      offset = 0;
+	  for (bsi = bsi_start (bb); !bsi_end_p (bsi); bsi_next (&bsi))
+	    {
+	      tree stmt = bsi_stmt (bsi);
+	      if (EXPR_HAS_LOCATION (stmt))
+		output_location (EXPR_FILENAME (stmt), EXPR_LINENO (stmt),
+				 &offset, bb);
+	    }
 
-	      /* We are looking for line number notes.  Search backward
-		 before basic block to find correct ones.  */
-	      insn = prev_nonnote_insn (insn);
-	      if (!insn)
-		insn = get_insns ();
-	      else
-		insn = NEXT_INSN (insn);
+	  /* Notice GOTO expressions we eliminated while constructing the
+	     CFG.  */
+	  if (single_succ_p (bb) && single_succ_edge (bb)->goto_locus)
+	    {
+	      /* ??? source_locus type is marked deprecated in input.h.  */
+	      source_locus curr_location = single_succ_edge (bb)->goto_locus;
+	      /* ??? The FILE/LINE API is inconsistent for these cases.  */
+#ifdef USE_MAPPED_LOCATION 
+	      output_location (LOCATION_FILE (curr_location),
+			       LOCATION_LINE (curr_location), &offset, bb);
+#else
+	      output_location (curr_location->file, curr_location->line,
+			       &offset, bb);
+#endif
+	    }
 
-	      while (insn != BB_END (bb))
-		{
-		  if (NOTE_P (insn))
-		    {
-		      /* Must ignore the line number notes that
-			 immediately follow the end of an inline function
-			 to avoid counting it twice.  There is a note
-			 before the call, and one after the call.  */
-		      if (NOTE_LINE_NUMBER (insn)
-			  == NOTE_INSN_REPEATED_LINE_NUMBER)
-			ignore_next_note = 1;
-		      else if (NOTE_LINE_NUMBER (insn) <= 0)
-			/*NOP*/;
-		      else if (ignore_next_note)
-			ignore_next_note = 0;
-		      else
-			{
-		          expanded_location s;
-		          NOTE_EXPANDED_LOCATION (s, insn);
-			  output_location (s.file, s.line, &offset, bb);
-			}
-		    }
-		  insn = NEXT_INSN (insn);
-		}
-
-	      if (offset)
-		{
-		  /* A file of NULL indicates the end of run.  */
-		  gcov_write_unsigned (0);
-		  gcov_write_string (NULL);
-		  gcov_write_length (offset);
-		}
+	  if (offset)
+	    {
+	      /* A file of NULL indicates the end of run.  */
+	      gcov_write_unsigned (0);
+	      gcov_write_string (NULL);
+	      gcov_write_length (offset);
 	    }
 	}
-      else
-	{
-	  gcov_position_t offset;
-
-	  FOR_EACH_BB (bb)
-	    {
-	      block_stmt_iterator bsi;
-
-	      offset = 0;
-
-	      if (bb == ENTRY_BLOCK_PTR->next_bb)
-		{
-		  expanded_location curr_location = 
-		    expand_location (DECL_SOURCE_LOCATION
-				     (current_function_decl));
-		  output_location (curr_location.file, curr_location.line,
-				   &offset, bb);
-		}
-
-	      for (bsi = bsi_start (bb); !bsi_end_p (bsi); bsi_next (&bsi))
-		{
-		  tree stmt = bsi_stmt (bsi);
-		  if (EXPR_HAS_LOCATION (stmt))
-		    output_location (EXPR_FILENAME (stmt), 
-				     EXPR_LINENO (stmt),
-				     &offset, bb);
-		}
-
-	      /* Notice GOTO expressions we eliminated while constructing the
-		 CFG.  */
-	      if (single_succ_p (bb) && single_succ_edge (bb)->goto_locus)
-		{
-		  /* ??? source_locus type is marked deprecated in input.h.  */
-		  source_locus curr_location = single_succ_edge (bb)->goto_locus;
-		  /* ??? The FILE/LINE API is inconsistent for these cases.  */
-#ifdef USE_MAPPED_LOCATION 
-		  output_location (LOCATION_FILE (curr_location),
-				   LOCATION_LINE (curr_location),
-				   &offset, bb);
-#else
-		  output_location (curr_location->file, curr_location->line,
-				   &offset, bb);
-#endif
-		}
-
-	      if (offset)
-		{
-		  /* A file of NULL indicates the end of run.  */
-		  gcov_write_unsigned (0);
-		  gcov_write_string (NULL);
-		  gcov_write_length (offset);
-		}
-	    }
-	 }
     }
 
   ENTRY_BLOCK_PTR->index = ENTRY_BLOCK;
@@ -1138,26 +1043,12 @@ branch_prob (void)
 	instrument_values (values);
 
       /* Commit changes done by instrumentation.  */
-      if (ir_type ())
-	bsi_commit_edge_inserts ();
-      else
-	{
-          commit_edge_insertions_watch_calls ();
-	  allocate_reg_info (max_reg_num (), FALSE, FALSE);
-	}
+      bsi_commit_edge_inserts ();
     }
 
   free_aux_for_edges ();
 
-  if (!ir_type ())
-    {
-      /* Re-merge split basic blocks and the mess introduced by
-	 insert_insn_on_edge.  */
-      cleanup_cfg (profile_arc_flag ? CLEANUP_EXPENSIVE : 0);
-      if (profile_dump_file())
-	dump_flow_info (profile_dump_file());
-    }
-
+  VEC_free (histogram_value, heap, values);
   free_edge_list (el);
   if (flag_branch_probabilities)
     profile_status = PROFILE_READ;
@@ -1341,45 +1232,3 @@ tree_register_profile_hooks (void)
   profile_hooks = &tree_profile_hooks;
 }
 
-
-/* Do branch profiling and static profile estimation passes.  */
-static void
-rest_of_handle_branch_prob (void)
-{
-  struct loops loops;
-
-  /* Discover and record the loop depth at the head of each basic
-     block.  The loop infrastructure does the real job for us.  */
-  flow_loops_find (&loops);
-
-  if (dump_file)
-    flow_loops_dump (&loops, dump_file, NULL, 0);
-
-  /* Estimate using heuristics if no profiling info is available.  */
-  if (flag_guess_branch_prob
-      && profile_status == PROFILE_ABSENT)
-    estimate_probability (&loops);
-
-  flow_loops_free (&loops);
-  free_dominance_info (CDI_DOMINATORS);
-}
-
-struct tree_opt_pass pass_branch_prob =
-{
-  "bp",                                 /* name */
-  NULL,                                 /* gate */   
-  rest_of_handle_branch_prob,           /* execute */       
-  NULL,                                 /* sub */
-  NULL,                                 /* next */
-  0,                                    /* static_pass_number */
-  TV_BRANCH_PROB,                       /* tv_id */
-  0,                                    /* properties_required */
-  0,                                    /* properties_provided */
-  0,                                    /* properties_destroyed */
-  0,                                    /* todo_flags_start */
-  TODO_dump_func,                       /* todo_flags_finish */
-  'b'                                   /* letter */
-};
-
-
-
