@@ -1,6 +1,6 @@
 // 2003-11-09 Paolo Carlini <pcarlini@suse.de>
 
-// Copyright (C) 2003 Free Software Foundation
+// Copyright (C) 2003, 2004 Free Software Foundation
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -15,7 +15,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 22.2.6.2.1 money_put members
@@ -36,7 +36,8 @@ void test01()
   
   // cache the money_put facet
   wostringstream oss;
-  const money_put<wchar_t>& mon_put = use_facet<money_put<wchar_t> >(oss.getloc()); 
+  const money_put<wchar_t>& mon_put =
+    use_facet<money_put<wchar_t> >(oss.getloc()); 
 
   iterator_type os_it01 = mon_put.put(oss.rdbuf(), true, oss, L' ', amount);
   wstring result = oss.str();
