@@ -1,4 +1,6 @@
-// Copyright (C) 2003 Free Software Foundation, Inc.
+// { dg-require-namedlocale "" }
+
+// Copyright (C) 2003, 2005 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -13,7 +15,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 27.8.1.4 Overridden virtual functions
@@ -22,25 +24,27 @@
 #include <locale>
 #include <testsuite_hooks.h>
 
-void test01()
+void test02()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
 
   wfilebuf fb;
-  locale loc(__gnu_test::try_named_locale("en_US.UTF-8"));
+  locale loc(locale("en_US.UTF-8"));
   fb.pubimbue(loc);
   fb.pubsetbuf(0, 0);
-  fb.open("tmp_11305-1", ios_base::out);
+  fb.open("tmp_11305-2", ios_base::out);
   wfilebuf::int_type n1 = fb.sputc(0x20000000);
+  wfilebuf::int_type n2 = fb.sputc(0x40000000);
   wfilebuf* f = fb.close();
   
   VERIFY( n1 != wfilebuf::traits_type::eof() );
+  VERIFY( n2 != wfilebuf::traits_type::eof() );
   VERIFY( f != NULL );
 }
 
 int main()
 {
-  test01();
+  test02();
   return 0;
 }
