@@ -38,28 +38,42 @@ void test01()
 	     traits_type::state_type>);
 
   basic_filebuf<pod_uchar> fb;
-  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
-  fb.open(name, ios_base::in | ios_base::out | ios_base::trunc);
-  fb.sputc(pod_uchar::from<char>('b'));
+  fb.open(name, ios_base::out);
   fb.sputc(pod_uchar::from<char>(0xff));
-
-  // Check that seekoff sets the current state during output
-  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::cur);
+  fb.sputc(pod_uchar::from<char>(0));
+  fb.sputc(pod_uchar::from<char>(0));
   fb.sputc(pod_uchar::from<char>('a'));
-  fb.sputc(pod_uchar::from<char>(0xff));
-  fb.sputc(pod_uchar::from<char>(0));
-  fb.sputc(pod_uchar::from<char>(0));
+  fb.sputc(pod_uchar::from<char>('a'));
+  fb.sputc(pod_uchar::from<char>('a'));
+  fb.sputc(pod_uchar::from<char>('a'));
+  fb.close();
 
-  fb.pubseekoff(0, ios_base::beg);
+  fb.open(name, ios_base::in);
   fb.sbumpc();
   fb.sbumpc();
+  fb.sbumpc();
+
+  // Check that seekoff resets the state when seeking to beginning.
+  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::beg);
+  VERIFY( pos != traits_type::pos_type(traits_type::off_type(-1)) );
 
   traits_type::int_type c = fb.sbumpc();
   VERIFY( c != traits_type::eof() );
+  VERIFY( traits_type::eq(traits_type::to_char_type(c),
+			  pod_uchar::from<char>(0xff)) );
+  fb.sbumpc();
+  fb.sbumpc();
+  fb.sbumpc();
+  fb.sbumpc();
+  fb.sbumpc();
+  c = fb.sbumpc();
+  VERIFY( c != traits_type::eof() );
   VERIFY( traits_type::eq(traits_type::to_char_type(c),
 			  pod_uchar::from<char>('a')) );
+  c = fb.sbumpc();
+  VERIFY( c == traits_type::eof() );
 
   fb.close();
 }
