@@ -1,4 +1,4 @@
-// 2006-07-15  Paolo Carlini  <pcarlini@suse.de>
+// 2006-10-12  Paolo Carlini  <pcarlini@suse.de>
 
 // Copyright (C) 2006 Free Software Foundation
 //
@@ -31,12 +31,14 @@ void test01()
   bool test __attribute__((unused)) = true;
 
   wostringstream oss_01;
-  const string str_01(5000000, 'a');
 
-  oss_01 << str_01.c_str();
+  oss_01.width(5000000);
+  const streamsize width = oss_01.width();
+
+  oss_01 << L'a';
 
   VERIFY( oss_01.good() );
-  VERIFY( oss_01.str().size() == str_01.size() );
+  VERIFY( oss_01.str().size() == width );
 }
 
 int main()
